Scriptname PA_POP_Parade extends Quest Conditional

PA_POP_ParadeIntro Property StarterScript Auto
PA_POP_Event_Parade Property EventScript Auto

Bool Property ShockComment Auto Conditional Hidden
Bool Property CanKill = False Auto Conditional Hidden

PA_POP_Scanner Property Scanner Auto

ReferenceAlias Property Torturer Auto
ReferenceAlias Property Guard Auto
ReferenceAlias[] Property ParticipantAliases Auto
ReferenceAlias[] Property SexAliases Auto
ReferenceAlias Property Winner Auto


Int[] Property PrevRanks Auto Hidden
Form[] Property LeftWeapons Auto Hidden
Form[] Property RightWeapons Auto Hidden

Scene Property ChaseScene Auto
Scene Property ParadeFinishScene Auto
Scene Property ParadeSexScene Auto

Idle Property TakeIdle Auto
Idle Property GiveIdle Auto

Weapon Property Whip Auto

Actor Property PlayerRef Auto

SexLabFramework Property SL Auto

PR_API PR

Int Property CurrIndex Auto Conditional
Bool RandomShocks
Bool Property MoreActors Auto Hidden Conditional

Function AcquirePRAPI()
    CanKill = false
    PR = Quest.GetQuest("PR_Main") as PR_API
    PlayerRef = PlayerRef
    RegisterForModEvent("PR_CollarShockedActor", "OnCollarShock")
EndFunction

Function SetLocation(ObjectReference target)
    CanKill = true
    int result = PR.ActivateMode(PlayerRef, target, 1, 5, 2.0)
    Debug.Trace("PR Activation Result: " + result)
EndFunction

Function ClearLocation()
    PR.ClearTarget(PlayerRef)
EndFunction

Function Shock()
    PR.Shock(PlayerRef, 0, 10)
    Utility.Wait(1.0)
    PlayerRef.ModAV("Health", 10)
EndFunction

Function OnCollarShock(Form akActor)
    Debug.Trace("Player was shocked")
    ShockComment = true
EndFunction

Function PrepChase()
    Debug.Trace("Finishing scene")

    Actor[] participants = Scanner.GetParticipants()
    
    PrevRanks = Utility.CreateIntArray(participants.length)
    LeftWeapons = Utility.CreateFormArray(participants.length)
    RightWeapons = Utility.CreateFormArray(participants.length)

    Debug.Trace("Parts: " + participants)

    int i = 0

    while i < participants.length
        ParticipantAliases[i].ForceRefTo(participants[i])
        if participants[i] as Actor && i > 0
            MoreActors = true
        endIf
        i += 1
    endWhile

    ChaseScene.Start()
EndFunction

Function GiveAllWhips()
    Torturer.GetActorRef().PlayIdle(GiveIdle)
    Utility.Wait(0.5)
    int i = 0
    while i < ParticipantAliases.Length
        Actor part = ParticipantAliases[i].GetRef() as Actor
        if part
            part.PlayIdle(TakeIdle)
            part.AddItem(Whip)
        endIf
        i += 1
    endWhile
EndFunction

Function StartChase()
    Debug.Trace("Starting chase")

    ClearLocation()
    Game.SetPlayerAIDriven(false)
    Game.EnablePlayerControls()

    int i = 0
    while i < ParticipantAliases.Length
        Actor part = ParticipantAliases[i].GetRef() as Actor
        if part
            PrevRanks[i] = part.GetRelationshipRank(PlayerRef)
            part.SetRelationshipRank(PlayerRef, -4)

            LeftWeapons[i] = part.GetEquippedWeapon(true)
            RightWeapons[i] = part.GetEquippedWeapon()

            part.RemoveItem(LeftWeapons[i])
            part.RemoveItem(RightWeapons[i])
            part.EquipItem(Whip)
            part.DrawWeapon()
            part.StartCombat(PlayerRef)
        endIf
        i += 1
    endWhile
    
    RandomShocks = true
    RegisterForSingleUpdate(2.5)
EndFunction

Event OnUpdate()
    if PlayerRef.GetActorValuePercentage("Health") < 0.5
        StartWinnerScene(none)
    elseIf RandomShocks
        if Utility.RandomFloat(0, 1.0) < 0.5
            Game.DisablePlayerControls()
            PR.Shock(PlayerRef, 0, 5)
            Utility.Wait(1.5)
            Game.EnablePlayerControls()
        endIf
    
        RegisterForSingleUpdate(2.5)
    endIf
EndEvent

Function StartWinnerScene(ObjectReference akAggressor)
    RandomShocks = false
    int i = 0
    while i < ParticipantAliases.Length
        Actor part = ParticipantAliases[i].GetActorRef()

        if part 
            if part == akAggressor
                Winner.ForceRefTo(part)
            endIf

            if part.IsInCombat()
                part.StopCombat()
            endIf
        endIf
        i += 1
    endWhile

    if !Winner.GetRef()
        Winner.ForceRefTo(ParticipantAliases[0].GetRef())
    endIf

    Game.DisablePlayerControls()
    Debug.SendAnimationEvent(PlayerRef, "ZapWriPose14")
    Debug.Trace("Ending chase w/ winner = " + Winner)
    ParadeFinishScene.Start()
EndFunction

Function WinnerSex()
    RandomShocks = false
    Game.EnablePlayerControls()
    Game.SetPlayerAIDriven(false)
    Debug.Trace("Starting scene between: " + PlayerRef + " " + Winner.GetRef())

    if !SL.QuickStart(PlayerRef, Winner.GetActorRef(), Victim=PlayerRef, AnimationTags="Aggressive")
        sslThreadController thread = SL.QuickStart(PlayerRef, Winner.GetActorRef(), Victim=PlayerRef)
        StartCrowd(thread)
    endIf
EndFunction

Function GangBang()
    Debug.Trace("Starting gangbang")
    currIndex = 0
    Shuffle()
    ParadeSexScene.Start()
EndFunction

Function Shuffle()
    Debug.Trace("Starting sex scene")
    SexAliases[0].ForceRefTo(ParticipantAliases[currIndex].GetRef())
    
    if !SexAliases[0].GetRef() || SexAliases[0].GetRef() == Winner.GetRef()
        OnSceneEnded()
    endIf

    int i = 0
    int j = 1
    while i < ParticipantAliases.Length
        if ParticipantAliases[i].GetRef() != SexAliases[0].GetRef()
            SexAliases[j].ForceRefTo(ParticipantAliases[i].GetRef())
            j += 1
        endIf

        i += 1
    endWhile

    i = 0
    while i < SexAliases.Length
        Debug.Trace("Sex Alias " + i + ":" + SexAliases[i].GetActorRef())
        Debug.Trace("Part Alias " + i + ":" + ParticipantAliases[i].GetActorRef())
        
        i += 1
    endWhile

EndFunction 

Function StartSex()
    Actor part = ParticipantAliases[currIndex].GetActorRef()

    Game.EnablePlayerControls()
    Game.SetPlayerAIDriven(false)

    if part
        sslThreadController thread = SL.QuickStart(PlayerRef, part, Victim = PlayerRef, AnimationTags = "Aggressive")
        StartCrowd(thread)
    endIf
EndFunction

Function StartCrowd(sslThreadController thread)
    if thread
        int handle = ModEvent.Create("CC_StartCrowd")

        if (handle)
            ModEvent.PushInt(handle, thread.tid) ; tid = thread id of the scene you want the crowd for
            ModEvent.Send(handle)
        endIf
    endIf
EndFunction

Event OnSceneEnded()
    Debug.Trace("Sex scene ended")
    currIndex += 1
    Debug.SendAnimationEvent(PlayerRef, "ZapWriPose14")
    if currIndex < ParticipantAliases.Length
        Shuffle()
    else
        Utility.Wait(3.0)
        Finish()
    endIf
EndEvent

Function Finish()
    EventScript.Blackfade()

    ParadeSexScene.Stop()

    RandomShocks = false
    Debug.MessageBox("After several more rounds of abuse, you finally pass out.")
    
    Torturer.GetRef().Delete()
    Guard.GetRef().Delete()
    Winner.Clear()

    int i = 0
    while i < ParticipantAliases.Length
        Actor part = ParticipantAliases[i].GetRef() as Actor
        
        if part
            part.AddItem(LeftWeapons[i])
            part.AddItem(RightWeapons[i])
            part.RemoveItem(Whip)
            part.SetRelationshipRank(PlayerRef, PrevRanks[i])
        endIf
        ParticipantAliases[i].Clear()

        i += 1
    endWhile

    StarterScript.UnequipAll(PlayerRef)
    StarterScript.RemoveTats(PlayerRef)
    
    Reset()

    EventScript.EventFinished(true)
EndFunction

